/* $Header: /src/carson/nistnet/RCS/hitbox.c,v 1.2 1997/10/20 17:11:39 carson Exp $ */

/* hitbox.c - A simple command-line interface for controlling the
 * "nistnet" network emulation kernel module.
 *
 * Usage: hitbox <options> - must be root to run it.
 *	-u			up (on)
 *	-d			down (off)
 *	-a src dest delay delsigma bandwidth drop dup drdmin drdmax	add new
 *	-r src dest		remove
 *	-s src dest		see stats
 *	-S src dest		see stats continuously
 *	-R			read table
 *	-D			debug on
 *	-U			debug off
 *	-G			global stats (time only)
 */
#include <stdio.h>
#include <stdlib.h>
#include <linux/ioctl.h>
#include <fcntl.h>
#include <linux/socket.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <math.h>
#include <time.h>

#include "nistnet.h"

int hitfd;

addhit(char *src, char *dest, int delay, int delsigma, int bandwidth, int drop, int dup, int drdmin, int drdmax)
{
	struct lin_hitreq addme;
	struct hostent *hent;
	struct in_addr one, two;
	char onebuf[20], twobuf[20];
	int ret;

	bzero(&addme, sizeof(addme));
	addme.delay = delay;
	addme.delsigma = delsigma;
	addme.bandwidth = bandwidth;
	addme.p_drop = drop;
	addme.p_dup = dup;
	addme.drd_min = drdmin;
	addme.drd_max = drdmax;
	if (isdigit(*src))
		addme.src = inet_addr(src);
	else {
		hent = gethostbyname(src);
		if (!hent) {
			fprintf(stderr, "Unknown host: %s\n", src);
			return;
		}
		addme.src = *(__u32 *)hent->h_addr;
	}
	if (isdigit(*dest))
		addme.dest = inet_addr(dest);
	else {
		hent = gethostbyname(dest);
		if (!hent) {
			fprintf(stderr, "Unknown host: %s\n", dest);
			return;
		}
		addme.dest = *(__u32 *)hent->h_addr;
	}
	one.s_addr = addme.src;
	two.s_addr = addme.dest;
	strcpy(onebuf, inet_ntoa(one));
	strcpy(twobuf, inet_ntoa(two));
	printf("addhit %s to %s, delay %d, delsigma %d, bandwidth %d, drop %d, dup %d, drdmin %d, drdmax %d\n",
		onebuf, twobuf, delay, delsigma, bandwidth, drop, dup, drdmin, drdmax);
	ret = ioctl(hitfd, HITIOCTL_ADD, &addme);
	if (ret < 0)
		perror("add");
}

rmhit(char *src, char *dest)
{
	struct lin_hitreq rmme;
	struct hostent *hent;
	struct in_addr one, two;
	char onebuf[20], twobuf[20];
	int ret;

	bzero(&rmme, sizeof(rmme));
	if (isdigit(*src))
		rmme.src = inet_addr(src);
	else {
		hent = gethostbyname(src);
		if (!hent) {
			fprintf(stderr, "Unknown host: %s\n", src);
			return;
		}
		rmme.src = *(__u32 *)hent->h_addr;
	}
	if (isdigit(*dest))
		rmme.dest = inet_addr(dest);
	else {
		hent = gethostbyname(dest);
		if (!hent) {
			fprintf(stderr, "Unknown host: %s\n", dest);
			return;
		}
		rmme.dest = *(__u32 *)hent->h_addr;
	}
	one.s_addr = rmme.src;
	two.s_addr = rmme.dest;
	strcpy(onebuf, inet_ntoa(one));
	strcpy(twobuf, inet_ntoa(two));
	printf("rmhit %s %s\n",
		onebuf, twobuf);
	ret = ioctl(hitfd, HITIOCTL_REMOVE, &rmme);
	if (ret < 0)
		perror("remove");
}

stathit(char *src, char *dest, int loop)
{
	struct lin_hitstats statme;
	struct hostent *hent;
	struct in_addr one, two;
	char onebuf[20], twobuf[20];
	int ret;
	struct timespec sleeptime;

	bzero(&statme, sizeof(statme));
	if (isdigit(*src))
		statme.hitreq.src = inet_addr(src);
	else {
		hent = gethostbyname(src);
		if (!hent) {
			fprintf(stderr, "Unknown host: %s\n", src);
			return;
		}
		statme.hitreq.src = *(__u32 *)hent->h_addr;
	}
	if (isdigit(*dest))
		statme.hitreq.dest = inet_addr(dest);
	else {
		hent = gethostbyname(dest);
		if (!hent) {
			fprintf(stderr, "Unknown host: %s\n", dest);
			return;
		}
		statme.hitreq.dest = *(__u32 *)hent->h_addr;
	}
	one.s_addr = statme.hitreq.src;
	two.s_addr = statme.hitreq.dest;
	strcpy(onebuf, inet_ntoa(one));
	strcpy(twobuf, inet_ntoa(two));
	printf("statme %s %s\n",
		onebuf, twobuf);
	printf("n_drops rand_drops drd_drops  mem_drops  dups   last packet      size qsize bandwidth\n");
restat:
	ret = ioctl(hitfd, HITIOCTL_STATS, &statme);
	if (ret < 0)
		perror("stats");
	printf("%7d %10d %9d %9d %5d %10d.%06d  %5d %5d %8ld %8ld",
		statme.n_drops,
		statme.rand_drops,
		statme.drd_drops,
		statme.mem_drops,
		statme.dups,
		statme.last_packet.tv_sec,
		statme.last_packet.tv_usec,
		statme.current_size,
		statme.qlen,
		statme.current_bandwidth,
		statme.bytes_sent
		);
	if (loop) {
		printf("\r");
		fflush(stdout);
		sleeptime.tv_sec = 0;
		sleeptime.tv_nsec = 10000;
		nanosleep(&sleeptime, NULL);
		goto restat;
	} else
		printf("\n");
}

#define BIG_GUY	1000

readhit()
{
	struct addrpair bigguy[BIG_GUY];
	int count, i;
	struct in_addr one, two;
	char onebuf[20], twobuf[20];

	count = read(hitfd, (char *)bigguy, BIG_GUY*sizeof(struct addrpair));
	count /= sizeof(struct addrpair);
	for (i=0; i < count; ++i) {
		one.s_addr = bigguy[i].src;
		two.s_addr = bigguy[i].dest;
		strcpy(onebuf, inet_ntoa(one));
		strcpy(twobuf, inet_ntoa(two));
		printf("%s -> %s\n", onebuf, twobuf);
	}
}

main(int argc, char **argv)
{
	int ret, value=0, debugon=1, debugoff=0;
	double ddrop, ddup;
	int drop, dup;

	/* Usage: hitbox
		-u			up (on)
		-d			down (off)
		-a src dest delay delsigma bandwidth drop dup drdmin drdmax	add new
		-r src dest		remove
		-s src dest		see stats
		-S src dest		see stats continuously
		-R			read table
		-D			debug on
		-U			debug off
		-G			global stats (time only)
	 */

	hitfd = open(DEVHITBOX, O_RDWR);
	if (hitfd < 0) {
		perror(DEVHITBOX);
		exit(1);
	}
	while (--argc > 0) {
		++argv;
		if (argv[0][0] == '-') {
			switch(argv[0][1]) {
			case 'u':
				ret = ioctl(hitfd, HITIOCTL_ON, &value);
				if (ret < 0)
					perror("on");
				break;
			case 'd':
				ret = ioctl(hitfd, HITIOCTL_OFF, &value);
				if (ret < 0)
					perror("off");
				break;
			case 'a':
				ddrop = atof(argv[6]);
				ddup = atof(argv[7]);
				printf("ddrop %f ddup %f\n", ddrop, ddup);
				drop = (ddrop*65536);
				dup = (ddup*65536);
				addhit(argv[1], argv[2], atoi(argv[3]), 
					atoi(argv[4]), atoi(argv[5]),
					drop, dup,
					atoi(argv[8]), atoi(argv[9]));
				argc -= 9;
				argv += 9;
				break;
			case 'r':
				rmhit(argv[1], argv[2]);
				argc -= 2;
				argv += 2;
				break;
			case 's':
				stathit(argv[1], argv[2], 0);
				argc -= 2;
				argv += 2;
				break;
			case 'S':
				stathit(argv[1], argv[2], 1);
				argc -= 2;
				argv += 2;
				break;
			case 'R':
				readhit();
				break;
			case 'D':
				ret = ioctl(hitfd, HITIOCTL_DEBUG, &debugon);
				if (ret < 0)
					perror("debug on");
				break;
			case 'U':
				ret = ioctl(hitfd, HITIOCTL_DEBUG, &debugoff);
				if (ret < 0)
					perror("debug off");
				break;
			case 'G':
				{
				struct lin_globalstats globs;
				unsigned long total;
				int i;

				ret = ioctl(hitfd, HITIOCTL_GLOBALSTATS, &globs);
				if (ret < 0)
					perror("global stats");
				else {
					total = 0;
					for (i=0; i < BAND_ARRAY; ++i)
						total +=
							globs.process_overhead[i];
					printf("Average processed packet overhead %ld usec\n",
						total/BAND_ARRAY);
					total = 0;
					for (i=0; i < BAND_ARRAY; ++i)
						total +=
							globs.unprocess_overhead[i];
					printf("Average unprocessed packet overhead %ld usec\n",
						total/BAND_ARRAY);
					total = 0;
					for (i=0; i < BAND_ARRAY; ++i)
						total +=
							globs.hash_tries[i];
					printf("Average hash tries %ld\n",
						total/BAND_ARRAY);
				}
				break;
				}
				default:
					fprintf(stderr, "Huh? what's %s?\n", argv[0]);
					break;
			}
		} else {	/* assume was add */
			ddrop = atof(argv[5]);
			ddup = atof(argv[6]);
			printf("ddrop %f ddup %f\n", ddrop, ddup);
			drop = (ddrop*65536);
			dup = (ddup*65536);
			addhit(argv[0], argv[1], atoi(argv[2]),
				atoi(argv[3]), atoi(argv[4]),
				drop, dup,
				atoi(argv[7]), atoi(argv[8]));
			argc -= 8;
			argv += 8;
		}
	}
}
